package jumpstart.min.business.domain.security;

import java.io.Serializable;
import java.util.Date;

import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.GeneratedValue;
import javax.persistence.GenerationType;
import javax.persistence.Id;
import javax.persistence.PostLoad;
import javax.persistence.PrePersist;
import javax.persistence.PreRemove;
import javax.persistence.PreUpdate;
import javax.persistence.Table;
import javax.persistence.Temporal;
import javax.persistence.TemporalType;
import javax.persistence.Transient;
import javax.persistence.UniqueConstraint;
import javax.persistence.Version;

import jumpstart.min.business.commons.exception.AuthenticationException;
import jumpstart.min.business.commons.exception.BusinessException;
import jumpstart.min.business.commons.exception.GenericBusinessException;
import jumpstart.min.business.commons.exception.ValueRequiredException;
import jumpstart.min.business.domain.base.BaseEntity;
import jumpstart.min.util.StringUtil;

/**
 * The User entity.
 */
@Entity
@Table(name = "user_min", uniqueConstraints = { @UniqueConstraint(columnNames = { "login_id" }) })
@SuppressWarnings("serial")
public class User extends BaseEntity {
	static public final String ADMIN_LOGINID = "admin";

	static public final String[] SALUTATIONS = { "", "Ms", "Mrs", "Mr", "Dr", "Prof" };

	@Id
	@GeneratedValue(strategy = GenerationType.AUTO)
	@Column(name = "id", nullable = false)
	protected Long id;

	@Version
	@Column(name = "version", nullable = false)
	protected Integer version;

	@Column(name = "login_id", length = 15, nullable = false)
	protected String loginId;

	@Column(name = "password", length = 32)
	protected String password;

	@Column(name = "salutation", length = 6)
	protected String salutation;

	@Column(name = "first_name", length = 20, nullable = false)
	protected String firstName;

	@Column(name = "last_name", length = 20, nullable = false)
	protected String lastName;

	@Column(name = "email_address", length = 80)
	protected String emailAddress;

	@Column(name = "active")
	protected boolean active;

	@Column(name = "expiry_date")
	@Temporal(TemporalType.DATE)
	protected Date expiryDate;

	@Transient
	protected transient String _loaded_password = null;

	public String toString() {
		StringBuffer buf = new StringBuffer();
		buf.append("User: [");
		buf.append("id=" + id + ", ");
		buf.append("loginId=" + loginId + ", ");
		buf.append("password=" + password + ", ");
		buf.append("salutation=" + salutation + ", ");
		buf.append("firstName=" + firstName + ", ");
		buf.append("lastName=" + lastName + ", ");
		buf.append("emailAddress=" + emailAddress + ", ");
		buf.append("expiryDate=" + expiryDate + ", ");
		buf.append("active=" + active + ", ");
		buf.append("version=" + version);
		buf.append("]");
		return buf.toString();
	}

	// The need for an equals() method is discussed at http://www.hibernate.org/109.html

	@Override
	public boolean equals(Object obj) {
		return (obj == this) || (obj instanceof User) && getId() != null && obj != null ? ((User) obj).getId().equals(
				this.getId()) : super.equals(obj);
	}

	// The need for a hashCode() method is discussed at http://www.hibernate.org/109.html

	@Override
	public int hashCode() {
		return getId() == null ? super.hashCode() : getId().hashCode();
	}

	@Override
	public Serializable getIdForMessages() {
		return getId();
	}

	@PrePersist
	void prePersistOfUser() throws BusinessException {
		validateUser();
	}

	@PostLoad
	void postLoadOfUser() {
		_loaded_password = password;
	}

	@PreUpdate
	void preUpdateOfUser() throws BusinessException {
		validateUser();
	}

	@PreRemove
	void preRemoveOfUser() throws BusinessException {
		// Check business rules here, eg.
		// if (entity.getParts().size() > 0) {
		// throw new CannotDeleteIsNotEmptyException(this, id, "Part");
		// }

		// There's no need to remove me from the parent(s) - that's the caller's
		// responsibility (and for performance, it might not bother)
	}

	public void validateUser() throws BusinessException {

		// Validate syntax...

		if (StringUtil.isEmpty(loginId)) {
			throw new ValueRequiredException(this, "User_loginId");
		}

		if (StringUtil.isEmpty(firstName)) {
			throw new ValueRequiredException(this, "User_firstName");
		}

		if (StringUtil.isEmpty(lastName)) {
			throw new ValueRequiredException(this, "User_lastName");
		}

		if (StringUtil.isEmpty(password)) {
			throw new ValueRequiredException(this, "User_password");
		}

		// Validate semantics...

		if (expiryDate != null && loginId.equals(ADMIN_LOGINID)) {
			throw new GenericBusinessException("User_expirydate_not_permitted_for_user", new Object[] { ADMIN_LOGINID });
		}

	}

	public void authenticate(String password) throws AuthenticationException {

		if (password == null || !password.equals(this.password)) {
			throw new AuthenticationException("User_password_incorrect");
		}

	}

	/**
	 * This method provides a way for users to change their own passwords.
	 */
	void changePassword(String currentPassword, String newPassword) throws BusinessException {

		if (this.password != null && !currentPassword.equals(this.password)) {
			throw new AuthenticationException("User_password_incorrect");
		}

		setPassword(newPassword);
	}

	public Long getId() {
		return id;
	}

	public Integer getVersion() {
		return version;
	}

	public String getLoginId() {
		return loginId;
	}

	public void setLoginId(String loginId) {
		this.loginId = loginId;
	}

	/**
	 * This method provides a way for security officers to "reset" the password.
	 */
	void setPassword(String newPassword) throws BusinessException {

		// TODO - review. Do all tests of password size, content, history, etc
		// here...

		if (StringUtil.isEmpty(newPassword)) {
			throw new ValueRequiredException(this, "User_password");
		}

		if (_loaded_password != null && newPassword.equals(_loaded_password)) {
			throw new GenericBusinessException("User_newpassword_is_same");
		}

		// The following is not at all secure. Remember that this object will be
		// detached and returned to the web client, where this clear-text
		// password might become vulnerable.
		// Alternatives include encrypting the password before storing it,
		// saving it encrypted to a separate entity that is never detached, or
		// using an enterprise-strength security framework like JAAS, Acegi,
		// etc. MD5 encryption is planned for this example.

		this.password = newPassword;
	}

	public String getFirstName() {
		return firstName;
	}

	public void setFirstName(String firstName) {
		this.firstName = firstName;
	}

	public String getLastName() {
		return lastName;
	}

	public void setLastName(String lastName) {
		this.lastName = lastName;
	}

	public String getEmailAddress() {
		return emailAddress;
	}

	public void setEmailAddress(String emailAddress) {
		this.emailAddress = emailAddress;
	}

	public boolean isActive() {
		return active;
	}

	public void setActive(boolean active) {
		this.active = active;
	}

	public String getSalutation() {
		return salutation;
	}

	public void setSalutation(String salutation) {
		this.salutation = salutation;
	}

	public Date getExpiryDate() {
		return expiryDate;
	}

	public void setExpiryDate(Date expiryDate) {
		this.expiryDate = expiryDate;
	}

}
